# coding: utf-8

from __future__ import absolute_import, unicode_literals

from django.conf import settings
_DEFAULT_CLOUD_STEPS = 6
_DEFAULT_CLOUD_MIN_COUNT = 1

CLOUD_STEPS = getattr(settings, 'TAGTOOLS_CLOUD_STEPS', _DEFAULT_CLOUD_STEPS)
CLOUD_MIN_COUNT = getattr(settings, 'TAGTOOLS_CLOUD_MIN_COUNT', _DEFAULT_CLOUD_MIN_COUNT)



# -*- coding: utf8 -*-

from __future__ import unicode_literals

import math
import logging
_LOG = logging.getLogger(__name__)

from django.contrib.contenttypes.models import ContentType
from django.db.models import Count, Max, Min


class BaseTagCloud(object):
    """
    Calculates a django-tagging-style tag-cloud.

    Adapt set_tags() to a suitable tagging-module.
    """

    def __init__(self, model, steps=6, min_count=1, alphabetic=True):
        self.steps = steps
        self.min_count = min_count
        self.model = model
        self.alphabetic = True
        self.set_tags()
        if len(self.tags):
            self.find_min_max()
            self.thresholds = self.calculate_thresholds()

    def set_tags(self):
        """
        Adapted to django-taggit
        """
        raise NotImplementedError("This TagCloud has not been adapted to a model")

    def find_min_max(self):
        edges = self.queryset.aggregate(max=Max('count'), min=Min('count'))
        self.max_weight = edges['max']
        self.min_weight = edges['min']

    def calculate_thresholds(self):
        delta = (self.max_weight - self.min_weight) / float(self.steps)
        return [self.min_weight + i * delta for i in range(1, self.steps + 1)]

    def calculate_tag_weight(self, weight):
        """
        Logarithmic tag weight calculation is based on code from the
        `Tag Cloud`_ plugin for Mephisto, by Sven Fuchs.

        .. _`Tag Cloud`: http://www.artweb-design.de/projects/mephisto-plugin-tag-cloud
        """
        if self.max_weight == 1:
            return weight
        else:
            # Fuchs-method
            return math.log(weight) * self.max_weight / math.log(self.max_weight)

    def calculate_cloud(self):
        """
        Add a ``font_size`` attribute to each tag according to the
        frequency of its use, as indicated by its ``count``
        attribute.

        ``steps`` defines the range of font sizes - ``font_size`` will
        be an integer between 1 and ``steps`` (inclusive).
        """
        tags = []
        if len(self.tags):
            for tag in self.tags:
                font_set = False
                tag_weight = self.calculate_tag_weight(tag['count'])
                for i in range(self.steps):
                    if not font_set and tag_weight <= self.thresholds[i]:
                        tag['font_size'] = i + 1
                        font_set = True
                tags.append(tag)
        return tags


class TaggitCloud(BaseTagCloud):
    """
    Given a model with django-taggit's TaggableManager, calculates a
    django-tagging-style tag-cloud.
    """

    def set_tags(self):
        """
        Adapted to django-taggit
        """
        from taggit.models import Tag, TaggedItem
        ct = ContentType.objects.get_for_model(self.model)
        tagitems = TaggedItem.objects.filter(content_type=ct).values_list('tag_id',
                flat=True)
        if not tagitems.exists():
            # Nothing tagged, nothing to do, abort, abort!
            self.tags = []
            self.queryset = Tag.objects.none()
            return
        queryset = Tag.objects.filter(id__in=tagitems)
        queryset = queryset.annotate(count=Count('taggit_taggeditem_items'))
        queryset = queryset.filter(count__gte=self.min_count)
        if self.alphabetic:
            queryset = queryset.order_by('name')
        self.queryset = queryset
        self.tags = self.queryset.values('name', 'slug', 'count')



from __future__ import absolute_import

from tagtools.tagcloud import *
from tagtools.settings import *
from taggit.models import Tag

def get_tagcloud_for_model(model, steps=CLOUD_STEPS, min_count=CLOUD_MIN_COUNT):
    cloud_calculator = TaggitCloud(model, steps, min_count)
    return cloud_calculator.calculate_cloud()

def set_tags_for_model(tags, model):
    existing_tags = set(model.tags.all())
    current_tags = set()
    if not tags:
        model.tags.set()
    else:
        for tag in tags:
            t_obj, created = Tag.objects.get_or_create(name=tag)
            if created:
                # Make sure the slug is generated
                t_obj.save()
            current_tags.add(t_obj)
        if existing_tags != current_tags:
            model.tags.add(*tags)
            remove_tags = existing_tags - current_tags
            if remove_tags:
                model.tags.remove(*remove_tags)
    return model, existing_tags != current_tags



# coding: utf-8

from __future__ import absolute_import, unicode_literals

from django.views.generic import ListView
from django.contrib.contenttypes.models import ContentType
from django.http import Http404
from django.utils.translation import ugettext as _

from tagtools.tagcloud import *
from tagtools.settings import *

from taggit.models import Tag, TaggedItem

def _get_queryset_and_model(queryset_or_model):
    """
    Given a ``QuerySet`` or a ``Model``, returns a two-tuple of
    (queryset, model).

    If a ``Model`` is given, the ``QuerySet`` returned will be created
    using its default manager.
    """
    try:
        return queryset_or_model, queryset_or_model.model
    except AttributeError:
        return queryset_or_model._default_manager.all(), queryset_or_model


class ListTaggedView(ListView):

    def get_queryset(self):
        try:
            queryset_or_model = self.kwargs.pop('queryset_or_model')
        except KeyError:
            raise AttributeError(_('tagged_object_list must be called with a queryset or a model.'))
        try:
            tag = self.kwargs.pop('tag')
        except KeyError:
            raise AttributeError(_('tagged_object_list must be called with a tag.'))
        try:
            self.tag_instance = Tag.objects.get(slug=tag)
        except Tag.DoesNotExist:
            raise Http404(_('No Tag found matching "%s".') % tag)
        qs, model = _get_queryset_and_model(queryset_or_model)
        ct_model = ContentType.objects.get_for_model(model)
        ti_qs = TaggedItem.objects.filter(content_type=ct_model, tag=self.tag_instance)
        queryset = model.objects.filter(pk__in=[o.object_id for o in ti_qs])
        return queryset

    def get_context_data(self, **kwargs):
        context = super(ListTaggedView, self).get_context_data(**kwargs)
        context['tag'] = self.tag_instance
        return context
tagged_object_list = ListTaggedView.as_view()



#!/usr/bin/env python

from distutils.core import setup
from setuptools import find_packages

README_FILE = open('README.rst')
try:
    long_description = README_FILE.read()
finally:
    README_FILE.close()

packages = (
    'tagtools',
)

setup(
    name='django-tag-tools',
    version='0.2.0',
    packages=('tagtools',),
    include_package_data=True,
    zip_safe=False,
    platforms=['any'],
    description='Tag cloud for django-taggit.',
    author_email='kaleissin@gmail.com',
    author='kaleissin',
    long_description=long_description,
    url='https://github.com/kaleissin/django-tag-tools',
    classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Topic :: Software Development :: Libraries :: Application Frameworks',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Framework :: Django :: 1.8',
            'Framework :: Django :: 1.9',
    ]
)
